@@ -1,6 +1,6 @@
 /* -*-mode:c++; c-file-style: "gnu";-*- */
 /*
- *  $Id: Cgicc.cpp,v 1.27 2008/05/17 09:15:58 sebdiaz Exp $
+ *  $Id: Cgicc.cpp,v 1.30 2013/01/12 11:11:46 sebdiaz Exp $
  *
  *  Copyright (C) 1996 - 2004 Stephen F. Booth <sbooth@gnu.org>
  *                       2007 Sebastien DIAZ <sebastien.diaz@gmail.com>
@@ -188,7 +188,7 @@ cgicc::Cgicc::Cgicc(CgiInput *input)
   fFormData.reserve(20);
   fFormFiles.reserve(2);
 
-  parseFormInput(fEnvironment.getPostData());
+  parseFormInput(fEnvironment.getPostData(), fEnvironment.getContentType());
   parseFormInput(fEnvironment.getQueryString());
 }
 
@@ -203,7 +203,7 @@ cgicc::Cgicc::operator= (const Cgicc& cgi)
   fFormData.clear();
   fFormFiles.clear();
 
-  parseFormInput(fEnvironment.getPostData());
+  parseFormInput(fEnvironment.getPostData(), fEnvironment.getContentType());
   parseFormInput(fEnvironment.getQueryString());
   
   return *this;
@@ -240,7 +240,7 @@ cgicc::Cgicc::restore(const std::string& filename)
   fFormData.clear();
   fFormFiles.clear();
 
-  parseFormInput(fEnvironment.getPostData());
+  parseFormInput(fEnvironment.getPostData(), fEnvironment.getContentType());
   parseFormInput(fEnvironment.getQueryString());
 }
 
@@ -336,9 +336,9 @@ cgicc::Cgicc::findEntries(const std::string& param,
 }
 
 void
-cgicc::Cgicc::parseFormInput(const std::string& data)
+cgicc::Cgicc::parseFormInput(const std::string& data, const std::string &content_type)
 {
-  std::string content_type 	= fEnvironment.getContentType();
+  
   std::string standard_type	= "application/x-www-form-urlencoded";
   std::string multipart_type 	= "multipart/form-data";
 
@@ -356,19 +356,38 @@ cgicc::Cgicc::parseFormInput(const std::string& data)
 
     // Parse the data in one fell swoop for efficiency
     while(true) {
-      // Find the '=' separating the name from its value
-      pos = data.find_first_of('=', oldPos);
+      // Find the '=' separating the name from its value, also have to check for '&' as its a common misplaced delimiter but is a delimiter none the less
+      pos = data.find_first_of( "&=", oldPos);
       
       // If no '=', we're finished
       if(std::string::npos == pos)
 	break;
       
       // Decode the name
+	// pos == '&', that means whatever is in name is the only name/value
+      if( data.at( pos ) == '&' )
+	  {
+	  	const char * pszData = data.c_str() + oldPos;
+		while( *pszData == '&' ) // eat up extraneous '&'
+		{
+			++pszData; ++oldPos;
+		}
+		if( oldPos >= pos )
+		{ // its all &'s
+			oldPos = ++pos;
+			continue;
+		}
+		// this becomes an name with an empty value
+		name = form_urldecode(data.substr(oldPos, pos - oldPos));
+		fFormData.push_back(FormEntry(name, "" ) );
+		oldPos = ++pos;
+		continue;
+	  }
       name = form_urldecode(data.substr(oldPos, pos - oldPos));
       oldPos = ++pos;
       
-      // Find the '&' separating subsequent name/value pairs
-      pos = data.find_first_of('&', oldPos);
+      // Find the '&' or ';' separating subsequent name/value pairs
+      pos = data.find_first_of(';&', oldPos);
       
       // Even if an '&' wasn't found the rest of the string is a value
       value = form_urldecode(data.substr(oldPos, pos - oldPos));
